﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace Common
{
   public abstract class EnumWrapper<T> : IEnumWrapper
   {
      // Performance note: If performance is an issue due to slow overriding GetCaption methods, cache those values
      private IEnumerable<BindableEnum> _bindableEnums;
      private IEnumerable<FieldInfo> _fields;

      protected virtual string GetCaption(string fieldName)
      {
         return fieldName;
      }

      public IEnumerable<BindableEnum> BindableEnums
      {
         get
         {
            if (_bindableEnums == null)
            {
               _fields = (from x in typeof(T).GetFields()
                          where !x.IsSpecialName
                          select x).ToList();
               var list = new List<BindableEnum>();
               foreach (var member in _fields)
               {
                  var caption = GetCaption(member.Name);
                  if (!string.IsNullOrEmpty(caption))
                  {
                     var value = (T)member.GetValue(null);
                     list.Add(new BindableEnum(value, caption, list));
                  }
               }
               _bindableEnums = list;
            }
            return _bindableEnums;
         }
      }

      public string EnumToString(object value)
      {
         if (value.GetType() != typeof(T))
         {            throw new InvalidOperationException();         }
         return GetCaption(((T)value).ToString());
      }

      public object StringToEnum(string value)
      {
         var q = from x in _fields
                 where GetCaption(x.Name) == value
                 select Enum.Parse(typeof(T), x.Name, true);
         return q.FirstOrDefault();
      }
   }

}
