﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel.Composition;
using System.Linq;
using Common;
using System.Windows.Data;
using System.Collections.Generic;

namespace May_Samples
{
   public abstract class EnumToBindableBase : IValueConverter
   {
      [ImportMany]
      public IEnumerable<Lazy<IEnumWrapper, IEnumWrapperExportMetadata>> _enumWrappers;

      protected virtual bool SupportsConvertBack
      {
         get
         { return false; }
      }

      protected abstract object InternalConvert(IEnumWrapper enumWrapper, 
         Type enumType,object value, Type targetType, object parameter, 
         System.Globalization.CultureInfo culture);

      protected virtual object InternalConvertBack(IEnumWrapper enumWrapper,
         Type enumType, object value, Type targetType, object parameter,
         System.Globalization.CultureInfo culture)
      {
         throw new NotImplementedException();
      }

      private IEnumWrapper GetWrapper(Type enumType)
      {
         if (_enumWrappers == null)
         { CompositionInitializer.SatisfyImports(this); }
         var lazyWrapper = (from x in _enumWrappers
                            where x.Metadata.TargetType == enumType
                            select x).FirstOrDefault();
         IEnumWrapper wrapper = null;
         if (lazyWrapper != null)
         {
            wrapper = lazyWrapper.Value;
         }
         return wrapper;
      }

      public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
      {
         var enumType = value.GetType();
         return InternalConvert(GetWrapper(enumType), enumType, value, targetType, 
            parameter, culture);
      }

      public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
      {
         if (!SupportsConvertBack)
         {
            throw new NotImplementedException();
         }
         return InternalConvertBack(GetWrapper(targetType), targetType, value, targetType, 
            parameter, culture);
      }
   }
}
